/**********************************************************************
*  2005 Microchip Technology Inc.
*
* FileName:        oscillator_failure_trap.s
* Dependencies:    Header (*.inc/.h) files if applicable, see below
* Processor:       dsPIC30Fxxxx
* Compiler:        MPLAB C30 v1.32.00 or higher
* IDE:             MPLAB IDE v7.21 or later
* Dev. Board Used: dsPICDEM 1.1 Development Board
* Hardware Dependencies: None
*
* SOFTWARE LICENSE AGREEMENT:
* Microchip Technology Inc. (Microchip) licenses this software to you
* solely for use with Microchip dsPIC digital signal controller
* products. The software is owned by Microchip and is protected under
* applicable copyright laws.  All rights reserved.
*
* SOFTWARE IS PROVIDED AS IS.  MICROCHIP EXPRESSLY DISCLAIMS ANY
* WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
* BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* HV               09/30/05  First release of source file
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
* THIS CODE ASSUMES THAT THE FAIL-SAFE CLOCK MONITOR HAS BEEN
* TURNED ON WITHIN FOSC CONFIGURATION REGISTER !!
* P.S. main.c file for config macro examples.
**********************************************************************/

.include "p30fxxxx.inc"

.global __OscillatorFail                ;Provide global scope to the Oscillator
                                        ;Failure Trap Handler. This ensures that
                                        ;the linker will place the address of
                                        ;the routine in the Interrupt Vector
                                        ;Table

.section .ndata, data, near
flagPLLoutOfLock: .hword 0x0000

.section .text                  ;Code section in Program Memory

/*
;On encountering a clock failure, if the fail-safe clock monitor (FSCM)
;is enabled, device executes code using the FRC oscillator and code execution
;reaches the Oscillatore Fail trap
*/
__OscillatorFail:
        push.d  w0                      ;Save w0, w1, w2 and w3 on to stack
        push.d  w2                      ;For fast context saving use "push.s"
                                        ;However, push.s may be used only once
                                        ;in your project

        mov     OSCCON, WREG            ;Read OSCCON in to w0

        btss    w0, #LOCK               ;Test the PLL LOCK bit
        setm    flagPLLoutOfLock        ;If LOCK bit is clear then flag this to the
                                        ;user for diagnostic information

        bclr    w0, #CF                 ;Before clearing the OSCFAIL trap status flag
                                        ;we should clear the Clock Fail(CF) bit in
                                        ;OSCCONL

                                        ;Load w0 with a byte value to write to
                                        ;OSCCONL so that the CF bit is cleared
        mov     #OSCCONL, w1            ;Point w1 to OSCCONL byte
        disi    #4                      ;Disable interrupts for the next
                                        ;5 instruction cycles
        mov     #0x46, w2               ;Move first OSCCONL unlock code byte
                                        ;to w2
        mov     #0x57, w3               ;Move second OSCCONL unlock code byte
                                        ;to w3
        mov.b   w2, [w1]                ;Perform byte-wide move of w2 to OSCCONL
        mov.b   w3, [w1]                ;Perform byte-wide move of w2 to OSCCONL

        mov.b   w0, [w1]                ;Write parameter in w0 (lower byte)to
                                        ;OSCCONL within one instruction cycle
                                        ;of unlocking OSCCONL using MOV instruction

        bclr.b    INTCON1, #OSCFAIL     ;Clear the Oscillator Failure Trap
                                        ;flag bit


        pop.d   w2                      ;Restore w0, w1, w2 and w3 from stack
        pop.d   w0                      ;You may use pop.s for fast context saving
                                        ;However pop.s may only be used once
                                        ;in your project.
        retfie                          ;Return from interrupt



.end



/*
EOF
*/

